/**********************************************************************\
**
**                         Copyright (c) 2004-2017 by
**                          HENGSTLER GmbH, Aldingen
**
**      This  software  is furnished under a licence and may be used and
**      copied only in accordance with the terms  of  such  licence  and
**      with the inclusion of the above copyright notice.  This software
**      or any other copies thereof may not  be  provided  or  otherwise
**      made  available  to any other person.  No title to and ownership
**      of the software is hereby transferred.
**
**      The  information  in  this software is subject to change without
**      notice and should not be construed as a commitment by Hengstler.
**
**  Module:      C56SdkDemo05DlgMain.cpp
**  Facility:    S684002 - Hengstler C56 Printer Driver for Windows XP/7/8/10
**  Component:   C56 SDK Demo Application 05 (C56SdkDemo05)
**  Date         2017APR27
**  Environment: Windows XP/7/8/10
**  Compiler:    Microsoft Visual Studio 2015
**  Abstract:    Implementation of main-dialog of SDK C-56 Demo 05
**  Notes:       -
**
\**********************************************************************/

/*
** INCLUDES
*/

#include "stdafx.h"

#include <winspool.h>

#include "..\..\Libraries\C56SDK.h"  /* SDK */

#include "C56SdkDemo05.h"
#include "C56SdkDemo05DlgMain.h"

/*
** COMPILER
*/

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

/*
** MESSAGE MAP
*/

BEGIN_MESSAGE_MAP(CC56SdkDemo05DlgMain, CDialog)
   ON_WM_SYSCOMMAND()
   ON_WM_PAINT()
   ON_WM_QUERYDRAGICON()
	ON_WM_TIMER()
   //}}AFX_MSG_MAP
   ON_BN_CLICKED(IDC_BUTTON_SINGLE_PRINTOUT, OnBnClickedButtonSinglePrintout)
   ON_BN_CLICKED(IDC_BUTTON_AUTO_PRINTOUT,   OnBnClickedButtonAutoPrintout)

END_MESSAGE_MAP()

//******************************************************************************************
// METHOD:   CC56SdkDemo05DlgMain::CC56SdkDemo05DlgMain
// ABSTRACT: Object constructor
//******************************************************************************************

CC56SdkDemo05DlgMain::CC56SdkDemo05DlgMain
   (CWnd* pParent /*=NULL*/)
:  CDialog(CC56SdkDemo05DlgMain::IDD, pParent)
{
   //-- Set member icon
   m_hIcon = AfxGetApp()->LoadIcon(IDR_C56);
}

//******************************************************************************************
// METHOD:   CC56SdkDemo05DlgMain::DoDataExchange
// ABSTRACT: Called by the framework to exchange and validate dialog data
//******************************************************************************************

void CC56SdkDemo05DlgMain::DoDataExchange(CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
}

//******************************************************************************************
// METHOD:   CC56SdkDemo05DlgMain::OnInitDialog
// ABSTRACT: This member function is called in response to the WM_INITDIALOG message.
//******************************************************************************************

BOOL CC56SdkDemo05DlgMain::OnInitDialog
   ()
{
   //-- Initialize dialog
   CDialog::OnInitDialog();

   //-- Check IDM_ABOUTBOX for being in system menu
   ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
   ASSERT(IDM_ABOUTBOX < 0xF000);

   //-- Create a timer for update of status
   m_puiTimerUpdateStatus = this->SetTimer(1, 500, 0);

   //-- Invalidate job number 
   m_bJobNr = 0xFF;
 
 	//-- Initialize member variables 
   {
      this->m_strPrinter  = "";
      this->m_dwTicketNr        = 1;
      this->m_dwNrOfTickets     = 10000;
      this->m_dwIntervalMs      = 2500;
      this->m_fPrinting         = FALSE;
      this->m_strStatus = "\nPRINTING DISABLED";
      this->m_strError  = "-";

      //-- Load these dialog items
      this->LoadDlgItems();
	}

   //-- Set C-56 icon for dialog 
   SetIcon(m_hIcon, TRUE);   // Use large symbol verwenden
   SetIcon(m_hIcon, FALSE);  // Use small symbol 

   //-- Setup printer selection combo box with available C-56 printers
   { 
      HANDLE hEnumPrinter; // Handle for enumeration 

      //-- Start access SDK to go through all C-56 printers 
      if ((hEnumPrinter = C56Sdk_PmStartEnumPrinter()) != NULL) {

         LPSTR       lpszPrinter; // Name of printer
         CComboBox * pPrinterList;  // Pointer to combo box item

         //-- Get printer selection combo box item
         if (pPrinterList = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_LIST))) {

            //-- Clear printer selection combo box
            pPrinterList->ResetContent();

            //-- Go through all C-56 printers
            for ( lpszPrinter = C56Sdk_PmEnumPrinter(hEnumPrinter, TRUE)
                ; lpszPrinter != NULL
                ; lpszPrinter = C56Sdk_PmEnumPrinter(hEnumPrinter, FALSE)
                ) {

               //-- Add printer to printer selection combo box 
               pPrinterList->AddString(lpszPrinter);
            }

            //-- Select 1st printer in printer selection combo box
            pPrinterList->SetCurSel(0);
         }
         
         //-- End of enumeration, free handle
         C56Sdk_PmEndEnumPrinter(hEnumPrinter);
      } 
   }
      

  //-- Set dialog items
   this->SetDlgItems();

   //-- Initialization successfull, set focus to first item
   return TRUE;  
}

/******************************************************************************************\
** CLASS  CC56SdkDemo05DlgMain
** METHOD LoadDlgItems
\******************************************************************************************/

void CC56SdkDemo05DlgMain::LoadDlgItems
   ()
{
   //-- Get dialog item settings from ini file
   {
      this->m_strPrinter       = theApp.GetProfileString("CC56SdkDemo05DlgMain", "PrinterDriver",   "");
      this->m_dwTicketNr       = theApp.GetProfileInt   ("CC56SdkDemo05DlgMain", "TicketNr",         1);
      this->m_dwNrOfTickets    = theApp.GetProfileInt   ("CC56SdkDemo05DlgMain", "NrOfTickets",   1000);
      this->m_dwIntervalMs     = theApp.GetProfileInt   ("CC56SdkDemo05DlgMain", "IntervalMs",    2000);
   }
}

/******************************************************************************************\
** CLASS  CC56SdkDemo05DlgMain
** METHOD StoreDlgItems
\******************************************************************************************/

void CC56SdkDemo05DlgMain::StoreDlgItems
   ()
{
   //-- Write dialog item settings in ini file
   {
      theApp.WriteProfileString("CC56SdkDemo05DlgMain", "PrinterDriver", this->m_strPrinter);
      theApp.WriteProfileInt   ("CC56SdkDemo05DlgMain", "TicketNr",      this->m_dwTicketNr);
      theApp.WriteProfileInt   ("CC56SdkDemo05DlgMain", "NrOfTickets",   this->m_dwNrOfTickets);
      theApp.WriteProfileInt   ("CC56SdkDemo05DlgMain", "IntervalMs",    this->m_dwIntervalMs);
   }
}

/******************************************************************************************\
** CLASS  CC56SdkDemo05DlgMain
** METHOD SetDlgItems
\******************************************************************************************/

void CC56SdkDemo05DlgMain::SetDlgItems
   ()
{
   //-- Set dialog item: printer driver 
   {
      CComboBox       * pPrinterList = NULL;      // Pointer to combo box item
      
      //-- Get printer selection combo box item
      if (pPrinterList = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_LIST))) {
      
         int iSel; // Selected item

         //-- Select driver in combo box
         if ((iSel = pPrinterList->FindStringExact(0, this->m_strPrinter)) != CB_ERR)
            pPrinterList->SetCurSel(iSel);

         //-- Enable/disable window
         pPrinterList->EnableWindow(!this->m_fPrinting);
      }
   }

   //-- Setup dialog item: ticket number
   {
      CString strDlgItem;        // String containing dialog item data
      CEdit * pDlgItemEdit;      // Pointer to edit dialog item

      //-- Get edit field 
      if ((pDlgItemEdit = ((CEdit *) this->GetDlgItem(IDC_EDIT_TICKET_NR))) != NULL) {

         //-- Setup string with field
         strDlgItem.Format(_T("%d"), m_dwTicketNr);

         //-- Set edit field 
         pDlgItemEdit->SetWindowText(strDlgItem);

         //-- Enable/disable window
         pDlgItemEdit->EnableWindow(!this->m_fPrinting);
      }
   }

   //-- Setup dialog item: number of tickets to print
   {
      CString strDlgItem;        // String containing dialog item data
      CEdit * pDlgItemEdit;      // Pointer to edit dialog item

      //-- Get edit field 
      if ((pDlgItemEdit = ((CEdit *) this->GetDlgItem(IDC_EDIT_NR_OF_TICKETS))) != NULL) {

         //-- Setup string with field
         strDlgItem.Format(_T("%d"), m_dwNrOfTickets);

         //-- Set edit field 
         pDlgItemEdit->SetWindowText(strDlgItem);

         //-- Enable/disable window
         pDlgItemEdit->EnableWindow(!this->m_fPrinting);
      }
   }

   //-- Setup dialog item: print interval (MS)
   {
      CString strDlgItem;        // String containing dialog item data
      CEdit * pDlgItemEdit;      // Pointer to edit dialog item

      //-- Get edit field 
      if ((pDlgItemEdit = ((CEdit *) this->GetDlgItem(IDC_EDIT_INTERVAL_MS))) != NULL) {

         //-- Setup string with field
         strDlgItem.Format(_T("%d"), m_dwIntervalMs);

         //-- Set edit field 
         pDlgItemEdit->SetWindowText(strDlgItem);

         //-- Enable/disable window
         pDlgItemEdit->EnableWindow(!this->m_fPrinting);
      }
   }

   //-- Setup dialog item: print status
   {
      CString strDlgItem;        // String containing dialog item data
      CStatic * pDlgItemText;      // Pointer to Text dialog item

      //-- Get edit field 
      if ((pDlgItemText = ((CStatic *) this->GetDlgItem(IDC_STATIC_STATUS))) != NULL) {

         //-- Set edit field 
         pDlgItemText->SetWindowText(this->m_strStatus);

         //-- Enable/disable window
         pDlgItemText->EnableWindow(this->m_fPrinting);
      }
   }

   //-- Setup dialog item: print error
   {
      CString    strDlgItem;        // String containing dialog item data
      CStatic * pDlgItemText;      // Pointer to Text dialog item

      //-- Get edit field 
      if ((pDlgItemText = ((CStatic *) this->GetDlgItem(IDC_STATIC_ERROR))) != NULL) {

         //-- Set edit field 
         pDlgItemText->SetWindowText(this->m_strError);

         //-- Enable/disable window
         pDlgItemText->EnableWindow(this->m_fPrinting);
      }
   }

   //-- Setup button to start or stop automatic printing
   {
      CButton * pDlgItemButton;      // Pointer to button dialog item

      //-- Get button field 
      if ((pDlgItemButton = ((CButton *) this->GetDlgItem(IDC_BUTTON_AUTO_PRINTOUT))) != NULL) {

          //-- Set edit field 
          pDlgItemButton->SetWindowText(this->m_fPrinting ? "Stop &Automatic Ticket Printout" : "Start &Automatic Ticket Printout");
      }
   }

   //-- Setup button for single printing
   {
      CButton * pDlgItemButton;      // Pointer to button dialog item

      //-- Get button field 
      if ((pDlgItemButton = ((CButton *) this->GetDlgItem(IDC_BUTTON_SINGLE_PRINTOUT))) != NULL) {

          //-- Enable/disable window
          pDlgItemButton->EnableWindow(!this->m_fPrinting);
      }
   }
}

/******************************************************************************************\
** CLASS  CC56SdkDemo05DlgMain
** METHOD GetDlgItems
\******************************************************************************************/

void CC56SdkDemo05DlgMain::GetDlgItems
   ()
{
   //-- Get dialog item: printer driver 
   {
      CComboBox       * pPrinterList = NULL;      // Pointer to combo box item
      
      this->m_strPrinter = "";

      //-- Get printer selection combo box item
      if (pPrinterList = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_LIST))) {

         INT nIndex;

         //-- Get actual selected driver 
         if ((nIndex = pPrinterList->GetCurSel()) != LB_ERR)
            pPrinterList->GetLBText(nIndex, this->m_strPrinter);
      }
   }

   //-- Get dialog item: ticket number
   {
      CString strDlgItem;        // String containing dialog item data
      CEdit * pDlgItemEdit;      // Pointer to edit dialog item

      //-- Get edit field 
      if ((pDlgItemEdit = ((CEdit *) this->GetDlgItem(IDC_EDIT_TICKET_NR))) != NULL) {

          //-- Get edit field contents
          pDlgItemEdit->GetWindowText(strDlgItem);

          //-- Set ticket number 
          m_dwTicketNr = abs(atoi(strDlgItem.GetBuffer())); 
          strDlgItem.ReleaseBuffer();
      }
   }

   //-- Get dialog item: number of tickets to print
   {
      CString strDlgItem;        // String containing dialog item data
      CEdit * pDlgItemEdit;      // Pointer to edit dialog item

      //-- Get edit field 
      if ((pDlgItemEdit = ((CEdit *) this->GetDlgItem(IDC_EDIT_NR_OF_TICKETS))) != NULL) {

          //-- Get edit field contents
          pDlgItemEdit->GetWindowText(strDlgItem);

          //-- Set number of tickets to print
          m_dwNrOfTickets = abs(atoi(strDlgItem.GetBuffer())); 
          strDlgItem.ReleaseBuffer();
      }
   }

   //-- Get dialog item: print interval (MS)
   {
      CString strDlgItem;        // String containing dialog item data
      CEdit * pDlgItemEdit;      // Pointer to edit dialog item

      //-- Get edit field 
      if ((pDlgItemEdit = ((CEdit *) this->GetDlgItem(IDC_EDIT_INTERVAL_MS))) != NULL) {

          //-- Get edit field contents
          pDlgItemEdit->GetWindowText(strDlgItem);

          //-- Set print interval 
          m_dwIntervalMs = abs(atoi(strDlgItem.GetBuffer())); 
          strDlgItem.ReleaseBuffer();
      }
   }
}

//******************************************************************************************
// METHOD:   CC56SdkDemo05DlgMain::OnSysCommand
// ABSTRACT: The framework calls this member function when the user selects a command from 
//           the Control menu, or when the user selects the Maximize or the Minimize button.
//******************************************************************************************

void CC56SdkDemo05DlgMain::OnSysCommand
   (UINT nID, LPARAM lParam)
{
    //-- Store dialog items on close 
	if (nID == SC_CLOSE) {

      //-- Get dialog item settings 
      this->GetDlgItems();

      //-- Store these dialog items
      this->StoreDlgItems();
   }

   //-- Call original CDialog::OnSysCommand member function
   CDialog::OnSysCommand(nID, lParam);
}

//******************************************************************************************
// METHOD:   CC56SdkDemo05DlgMain::OnPaint
// ABSTRACT: The framework calls this member function when Windows or an application makes a 
//           request to repaint a portion of an application's window.
//******************************************************************************************

void CC56SdkDemo05DlgMain::OnPaint
   () 
{
   //-- Check if dialog is iconic
   if (IsIconic()) {

      CPaintDC dc(this); // Device context for paint 

      //-- Erase background of icon
      SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

      //-- Center symbol in clientr rectangle
      int cxIcon = GetSystemMetrics(SM_CXICON);
      int cyIcon = GetSystemMetrics(SM_CYICON);
      CRect rect;
      GetClientRect(&rect);
      int x = (rect.Width() - cxIcon + 1) / 2;
      int y = (rect.Height() - cyIcon + 1) / 2;

      //-- Draw symbol 
      dc.DrawIcon(x, y, m_hIcon);
   }
   else {

      //-- Call original CDialog::OnPaint member function
      CDialog::OnPaint();
   }
}

//******************************************************************************************
// METHOD:   CC56SdkDemo05DlgMain::OnQueryDragIcon
// ABSTRACT: The framework calls this member function by a minimized (iconic) window that 
//           does not have an icon defined for its class.
//******************************************************************************************

HCURSOR CC56SdkDemo05DlgMain::OnQueryDragIcon
   ()
{
   return static_cast<HCURSOR>(m_hIcon);
}

/******************************************************************************************\
** CLASS  CC56SdkDemo05DlgMain
** METHOD OnBnClickedButtonAutoPrintout
\******************************************************************************************/

void CC56SdkDemo05DlgMain::OnBnClickedButtonAutoPrintout
   ()
{
   //-- Toggle printing flag
   this->m_fPrinting = !this->m_fPrinting;

   //-- Get dialog item settings 
   this->GetDlgItems();

   //-- Correct data 
   m_dwIntervalMs = max(m_dwIntervalMs, 1);

   //-- Store these dialog items
   this->StoreDlgItems();

   //-- Update printing status / error
   this->m_strStatus = (this->m_fPrinting ? "\nPRINTING ENABLED" : "\nPRINTING DISABLED");  
   this->m_strError  = "-";  

   //-- Update dialog items
   this->SetDlgItems();

   //-- Kill timer and create new one if required
   this->KillTimer(2); this->m_puiTimerPrintout = 0;
   if (this->m_fPrinting)
      this->m_puiTimerPrintout = this->SetTimer(2, m_dwIntervalMs, NULL);

   //-- Set focus on start stop button
   this->GetDlgItem(IDC_BUTTON_AUTO_PRINTOUT)->SetFocus();
}

/******************************************************************************************\
** CLASS  CC56SdkDemo05DlgMain
** METHOD OnBnClickedButtonAutoPrintout
\******************************************************************************************/

void CC56SdkDemo05DlgMain::OnBnClickedButtonSinglePrintout
   ()
{
   //-- Get dialog item settings 
   this->GetDlgItems();

   //-- Store these dialog items
   this->StoreDlgItems();

   //-- Update printing status / error
   this->m_strStatus = "-";  
   this->m_strError  = "-";  

   //-- Do printout 
   this->DoPrintout(FALSE);

   //-- Update printing status / error
   this->m_strStatus = "\nPRINTING DISABLED";  
   // this->m_strError  = "-";  

   //-- Update dialog items
   this->SetDlgItems();
}

//******************************************************************************************
// METHOD:   CC56SdkDemo05DlgMain::DoUpdateStatus
// ABSTRACT: This member function is to update status information displayed 
//******************************************************************************************

VOID CC56SdkDemo05DlgMain::DoUpdateStatus
   (VOID)
{
   CComboBox * pPrinterList;  // Pointer to combo box item

   //-- Get printer list (dialog item combo box)
   if ((pPrinterList = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_LIST))) != NULL) {

      INT iPrinterSel; // Selected printer

      //-- Determine selected printer
      if ((iPrinterSel = pPrinterList->GetCurSel()) != CB_ERR) {

         LPSTR lpszPrinter = NULL;  // Printer name

         //-- Allocate memory for this printer and get name from combo box
         if (    ((lpszPrinter = (LPSTR) malloc(pPrinterList->GetLBTextLen(iPrinterSel)+1)) != NULL) 
              && (pPrinterList->GetLBText(iPrinterSel, lpszPrinter) != CB_ERR)
            ) {

            DWORD               dwStatus;  // Printer status
            DWORD               dwAgeMs;   // Age of status in ms
            C56_STATUS_RESPONSE eResponse; // Status response
            int                 siPhTempC; // Temperature of printhead in C

            //-- Get printer status 
            dwStatus = C56Sdk_PmGetPrinterStatus(lpszPrinter, &dwAgeMs);

            //-- Check age of status being < xxx ms
            //.. and expand status
            //.. and check power on flag, which must allways be raised
            if (     (dwAgeMs <= 10000)
                  && C56Sdk_GpExpandStatus(dwStatus, &eResponse, &siPhTempC, NULL, NULL)
                  && ((eResponse & C56SR_PRINTER_POWERON) != 0)
                ) {

               //-- Update printer status
               {  
                  //-- Update dialog items
                  this->GetDlgItem(IDC_RADIO_STATUS_TEMP_ALERT)->EnableWindow(TRUE);
                  this->CheckDlgButton(IDC_RADIO_STATUS_TEMP_ALERT, (eResponse & C56SR_PRINTER_TEMPALERT)    != 0);
                  this->GetDlgItem(IDC_RADIO_STATUS_LAST_PRINTOUT_LOST)->EnableWindow(TRUE);
                  this->CheckDlgButton(IDC_RADIO_STATUS_LAST_PRINTOUT_LOST, (eResponse & C56SR_PRINTER_PRINTOUTLOST) != 0);
                  this->GetDlgItem(IDC_RADIO_STATUS_PAPER_JAM)->EnableWindow(TRUE);
                  this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_JAM, (eResponse & C56SR_PRINTER_PAPERJAM)     != 0);
                  this->GetDlgItem(IDC_RADIO_STATUS_PAPER_LOW)->EnableWindow(TRUE);
                  this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_LOW, (eResponse & C56SR_PRINTER_PAPERLOW)     != 0);
                  this->GetDlgItem(IDC_RADIO_STATUS_PAPER_OUT)->EnableWindow(TRUE);
                  this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_OUT, (eResponse & C56SR_PRINTER_OUTOFPAPER)   != 0);
                  this->GetDlgItem(IDC_RADIO_STATUS_POWER_ON)->EnableWindow(TRUE);
                  this->CheckDlgButton(IDC_RADIO_STATUS_POWER_ON, (eResponse & C56SR_PRINTER_POWERON)      != 0);
               }

               //-- Update printer temperature
               {  
                  char szBuff[20]; // Buffer

                  //-- Update dialog items
                  sprintf_s(szBuff, sizeof(szBuff), "%-3.3d", siPhTempC);
                  this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C)->EnableWindow(TRUE);
                  this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C)->SetWindowText(szBuff);
                  this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C_TXT)->EnableWindow(TRUE);
                  sprintf_s(szBuff, sizeof(szBuff), "%-3.3d", (signed int) ((siPhTempC * 1.8) + 32));
                  this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F)->EnableWindow(TRUE);
                  this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F)->SetWindowText(szBuff);
                  this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F_TXT)->EnableWindow(TRUE);
               }

               //-- Update printer hex code
               {  
                  char szBuff[20]; // Buffer
                  
                  //-- Update dialog items
                  sprintf_s(szBuff, sizeof(szBuff), "[%2.2X][%2.2X][%2.2X][%2.2X]"
                         , (WORD) (((BYTE *) &dwStatus)[0])
                         , (WORD) (((BYTE *) &dwStatus)[1])
                         , (WORD) (((BYTE *) &dwStatus)[2])
                         , (WORD) (((BYTE *) &dwStatus)[3])
                         );
                  this->GetDlgItem(IDC_STATIC_STATUS_HEX)->EnableWindow(TRUE);
                  this->GetDlgItem(IDC_STATIC_STATUS_HEX)->SetWindowText(szBuff);
               }
            }
            else {

               //-- Invalidate Printer status
               {  
                  //-- Update dialog items
                  this->GetDlgItem(IDC_RADIO_STATUS_TEMP_ALERT)->EnableWindow(FALSE);
                  this->CheckDlgButton(IDC_RADIO_STATUS_TEMP_ALERT, FALSE);
                  this->GetDlgItem(IDC_RADIO_STATUS_LAST_PRINTOUT_LOST)->EnableWindow(FALSE);
                  this->CheckDlgButton(IDC_RADIO_STATUS_LAST_PRINTOUT_LOST, FALSE);
                  this->GetDlgItem(IDC_RADIO_STATUS_PAPER_JAM)->EnableWindow(FALSE);
                  this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_JAM, FALSE);
                  this->GetDlgItem(IDC_RADIO_STATUS_PAPER_LOW)->EnableWindow(FALSE);
                  this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_LOW, FALSE);
                  this->GetDlgItem(IDC_RADIO_STATUS_PAPER_OUT)->EnableWindow(FALSE);
                  this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_OUT, FALSE);
                  this->GetDlgItem(IDC_RADIO_STATUS_POWER_ON)->EnableWindow(FALSE);
                  this->CheckDlgButton(IDC_RADIO_STATUS_POWER_ON, FALSE);
               }

               //-- Invalidate printer temperature
               {  
                  //-- Update dialog items
                  this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C)->EnableWindow(FALSE);
                  this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C)->SetWindowText("-");
                  this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C_TXT)->EnableWindow(FALSE);
                  this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F)->EnableWindow(FALSE);
                  this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F)->SetWindowText("-");
                  this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F_TXT)->EnableWindow(FALSE);
               }
   
               //-- Invalidate printer hex code
               {  
                  //-- Update dialog items
                  this->GetDlgItem(IDC_STATIC_STATUS_HEX)->EnableWindow(FALSE);
                  this->GetDlgItem(IDC_STATIC_STATUS_HEX)->SetWindowText("[??][??][??][??]");
               }
            }

            //-- Update job number
            {
               CHAR szBuff[20];   // Buffer

               //-- Update dialog items
               this->GetDlgItem(IDC_STATIC_JOB_NR_TXT)->EnableWindow(m_bJobNr != 0xFF);
               this->GetDlgItem(IDC_STATIC_JOB_NR)->EnableWindow(m_bJobNr != 0xFF);
               sprintf_s(szBuff, sizeof(szBuff), m_bJobNr != 0xFF ? "%2.2d" : "-", m_bJobNr);
               this->GetDlgItem(IDC_STATIC_JOB_NR)->SetWindowText(szBuff);
            }

            //-- Update Job status
            {  
               C56_JOBSTATUS       eJobStatus;   // C56 Job status

               //-- Get Job status 
               eJobStatus = C56Sdk_PmGetJobStatus(lpszPrinter, m_bJobNr);

               //-- Update dialog items
               this->GetDlgItem(IDC_RADIO_STATUS_JOB_SPOOLER)->EnableWindow(m_bJobNr != 0xFF);
               this->CheckDlgButton(IDC_RADIO_STATUS_JOB_SPOOLER, (eJobStatus & (C56JS_SPOOLING | C56JS_SPOOLED)) != 0);
               this->GetDlgItem(IDC_RADIO_STATUS_JOB_SPOOLER)->SetWindowText
                             (eJobStatus & C56JS_SPOOLED  ? "Print job has been spooled"
                             :eJobStatus & C56JS_SPOOLING ? "...... SPOOLING ......"
                             :                              "Print job is to be spooled"
                             );
               this->GetDlgItem(IDC_RADIO_STATUS_JOB_PRINTER)->EnableWindow(m_bJobNr != 0xFF);
               this->CheckDlgButton(IDC_RADIO_STATUS_JOB_PRINTER, (eJobStatus & (C56JS_PRINTING | C56JS_PRINTED)) != 0);
               this->GetDlgItem(IDC_RADIO_STATUS_JOB_PRINTER)->SetWindowText
                             (eJobStatus & C56JS_PRINTED  ? "Print job has been printed"
                             :eJobStatus & C56JS_PRINTING ? "...... PRINTING ......"
                             :                              "Print job is to be printed"
                             );
               this->GetDlgItem(IDC_RADIO_STATUS_JOB_CONFIRMED)->EnableWindow(m_bJobNr != 0xFF);
               this->CheckDlgButton(IDC_RADIO_STATUS_JOB_CONFIRMED, (eJobStatus & C56JS_CONFIRMED) != 0);
            }                     
         }
 
         //-- Free allocated memory
         if (lpszPrinter) free(lpszPrinter); lpszPrinter = NULL; 
          
      }
   }    
}

//******************************************************************************************
// METHOD:   CC56SdkDemo05DlgMain::OnTimer
// ABSTRACT: This member function is called in response to the WM_TIMER message.
//******************************************************************************************

VOID CC56SdkDemo05DlgMain::OnTimer
   (UINT_PTR nIdEvent)
{
   //
   if (nIdEvent == m_puiTimerUpdateStatus) {
  
      this->DoUpdateStatus();
   }
   else 
   if (nIdEvent == m_puiTimerPrintout) {

      this->DoPrintout(TRUE);
   }
   else {
      //-- Other dialog timer
      CDialog::OnTimer(nIdEvent);

   }
}

//******************************************************************************************
// METHOD:   CC56SdkDemo05DlgMain::DoPrintout
// ABSTRACT: Handler for printout button being pressed
//******************************************************************************************

void CC56SdkDemo05DlgMain::DoPrintout
   (BOOL fAuto)
{
   //-- Reset printer status / format
   if (fAuto) {
      this->m_strStatus.Format("-");
      this->m_strError.Format("-");
   }

   //-- Check if there are still tickets to print
   if (!fAuto || (this->m_dwNrOfTickets > 0)) {
    
      //-- Single ticket or auto
      if (fAuto) {

         DWORD dwTimeToGo;                                  // Estimated time to go in sec

         //-- Determine time to go in sec
         dwTimeToGo = (this->m_dwNrOfTickets * max(1, m_dwIntervalMs)) / 1000;

         //-- Update printing status 
         this->m_strStatus.Format("PRINTING TICKET %lu\n%lu TICKETS TO PRINT\nESTIMATED TIME TO GO %2.2lu:%2.2lu:%2.2lu", this->m_dwTicketNr+1, this->m_dwNrOfTickets-1, dwTimeToGo / 3600, (dwTimeToGo % 3600) / 60, dwTimeToGo % 60);  
      }
      else {

         //-- Update printing status 
         this->m_strStatus.Format("\nPRINTING SINGLE TICKET\n");  
      }

      //-- Do not print if spooler is non empty
      if (C56Sdk_PmIsSpoolerEmpty((LPSTR) (LPCSTR) this->m_strPrinter)) {

         HANDLE hDocument; // Handle to document 
 
         //-- Start this document
         if ((hDocument = C56Sdk_DmStartDocument((LPSTR) (LPCSTR) this->m_strPrinter, "C56SdkDemo05")) != NULL) {
         
            //-- Add initial setttings
            C56Sdk_DmAddDocumentInitialSettings(hDocument);

            //-- Set initial linefeeds to power up the motor 
            C56Sdk_DcAddPrintMode(hDocument, C56PM_FONT_08X16);
            C56Sdk_DcAddLineFeed(hDocument, 1);

            //-- Add ticket header
            {                             
               C56Sdk_DcAddPrintMode(hDocument, C56PM_FONT_08X16);
               C56Sdk_DcAddLineSpacing(hDocument, 0);
               C56Sdk_DcAddText(hDocument, "=======================================================\n");
               C56Sdk_DcAddText(hDocument, "                   START OF TICKET\n");
               C56Sdk_DcAddText(hDocument, "-------------------------------------------------------\n");
               C56Sdk_DcAddText(hDocument, "                   C56 SDK DEMO 05\n");
               C56Sdk_DcAddText(hDocument, "-------------------------------------------------------\n");
  
               //-- Debug characters 
               // C56Sdk_DcAddText(hDocument, "00000000011111111112222222222333333333344444444445555555");
               // C56Sdk_DcAddText(hDocument, "12345678901234567890123456789012345678901234567890123456");
               // C56Sdk_DcAddText(hDocument, "-------------------------------------------------------\n");
            }

            //-- Add ticket info
            {         
               CString    oString;       // Data string
               CTime      oTime;         // Time object 
                  
               //-- Write actual date and time
               oTime = CTime::GetCurrentTime();
               oString.Format("    Date of printout  : %s\n", (LPSTR) (LPCSTR) oTime.Format( "%B %d, %Y"));
               C56Sdk_DcAddText(hDocument, (LPSTR) (LPCSTR)  oString);
               oString.Format("    Time of printout  : %s\n", (LPSTR) (LPCSTR) oTime.Format( "%H:%M:%S"));
               C56Sdk_DcAddText(hDocument, (LPSTR) (LPCSTR)  oString);

               //-- Check for auto printout and print ticket number if so
               if (fAuto) {

                  //-- Write ticket number 
                  oString.Format("    Ticket number     : %d\n", this->m_dwTicketNr);
                  C56Sdk_DcAddText(hDocument, (LPSTR) (LPCSTR)  oString);

                  //-- Write print intervall
                  oString.Format("    Print interval    : %d MS\n", this->m_dwIntervalMs);
                  C56Sdk_DcAddText(hDocument, (LPSTR) (LPCSTR)  oString);
               }

               //-- Add horizontal bar
               C56Sdk_DcAddText(hDocument, "=======================================================\n");
            }

            //-- Ticket data
            {
               //-- Set print mode .... 
               C56Sdk_DcAddPrintMode(hDocument, (C56_PRINT_MODE) (C56PM_FONT_08X16 | C56PM_DOUBLE_WIDTH | C56PM_DOUBLE_HEIGHT));
      
               //-- Set line pitch to ... 
               C56Sdk_DcAddLinePitch(hDocument, 32);
                   
               //-- Set tab stops to ... 
               {  
                  //C56Sdk_DcAddHorizontalTabPos(hDocument, 4, 0);
                    
                  unsigned char aubTabs[] = {0x1b, 0x44, 0x04, 0x00};
                  C56Sdk_DmAddDocumentData(hDocument, aubTabs, sizeof(aubTabs));
               }

               //-- Set linefeeds 
               C56Sdk_DcAddLineFeed(hDocument, 2);

               //-- Enable reverse printing 
               C56Sdk_DcAddEnableReversePrinting(hDocument);

               //-- Add black text 
               C56Sdk_DcAddText(hDocument, "\t                     \n");
               C56Sdk_DcAddText(hDocument, "\t  H E N G S T L E R  \n");
               C56Sdk_DcAddText(hDocument, "\t                     \n");

               //-- Enable reverse printing 
               C56Sdk_DcAddDisableReversePrinting(hDocument);

               //-- Set linefeeds 
               C56Sdk_DcAddLineFeed(hDocument, 2);

               //-- Add address
               C56Sdk_DcAddText(hDocument, "\tUhlandstrasse 49\n");
               C56Sdk_DcAddText(hDocument, "\t78554 Aldingen\n");
               C56Sdk_DcAddText(hDocument, "\tGermany\n");
               C56Sdk_DcAddText(hDocument, "\tTel. +49-(0)7424-890\n");
               C56Sdk_DcAddText(hDocument, "\tinfo@hengstler.com\n");
               C56Sdk_DcAddText(hDocument, "\twww.hengstler.com\n");

               //-- Set linefeeds 
               C56Sdk_DcAddLineFeed(hDocument, 2);

               //-- Add date/time
               {
                  CTime      oTime;         // Time object 
                  oTime = CTime::GetCurrentTime();
                  C56Sdk_DcAddText(hDocument, "\t");
                  C56Sdk_DcAddText(hDocument, (LPSTR) (LPCSTR)  oTime.Format( "%y/%m/%d"));
                  C56Sdk_DcAddText(hDocument, "       ");
                  C56Sdk_DcAddText(hDocument, (LPSTR) (LPCSTR)  oTime.Format( "%H:%M"));
                  C56Sdk_DcAddText(hDocument, "\n");
               }

               //-- Set linefeeds 
               C56Sdk_DcAddLineFeed(hDocument, 2);

               //-- Add ticket data
               {
                  C56Sdk_DcAddText(hDocument, "\t001 996278984 (1)\n");
                  C56Sdk_DcAddText(hDocument, "\t 2,00 * 4,00 99\n");
                  C56Sdk_DcAddText(hDocument, "\t 500 g./m2      8,00\n");

                  C56Sdk_DcAddText(hDocument, "\t001 996278982 (1)\n");
                  C56Sdk_DcAddText(hDocument, "\t 1,00 * 4,00 99\n");
                  C56Sdk_DcAddText(hDocument, "\t 500 g./m2     12,00\n");

                  C56Sdk_DcAddText(hDocument, "\t001 996278984 (1)\n");
                  C56Sdk_DcAddText(hDocument, "\t 2,00 * 4,00 99\n");
                  C56Sdk_DcAddText(hDocument, "\t 500 g./m2      8,00\n");

                  C56Sdk_DcAddText(hDocument, "\t            --------\n");
                  C56Sdk_DcAddText(hDocument, "\tTotal EUR      38,00\n");
               }     

               //-- Set linefeeds 
               C56Sdk_DcAddLineFeed(hDocument, 2);

               //-- Add barcode 
               {
                  unsigned char aubBarCode[] = {0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37};
                  C56Sdk_DcAddBarCodeWidth(hDocument, 2, 6);
                  C56Sdk_DcAddBarCodeHeight(hDocument, 60);
                  C56Sdk_DcAddBarCodeWidthMagnification(hDocument, 1);
                  C56Sdk_DcAddText(hDocument, "\t");
                  C56Sdk_DcAddBarCodePrint(hDocument, C56BT_CODE39, 8, aubBarCode, FALSE);
                  C56Sdk_DcAddText(hDocument, "\n");
               }

               //-- Set linefeeds 
               C56Sdk_DcAddLineFeed(hDocument, 2);
            }

            //-- Add ticket footer
            {                             
               C56Sdk_DcAddPrintMode(hDocument, C56PM_FONT_08X16);
               C56Sdk_DcAddLineSpacing(hDocument, 0);
               C56Sdk_DcAddText(hDocument, "=======================================================\n");
               C56Sdk_DcAddText(hDocument, "                    END OF TICKET\n");
               C56Sdk_DcAddText(hDocument, "=======================================================\n");
            }

            //-- Add final setttings
            C56Sdk_DmAddDocumentFinalSettings(hDocument);

            //-- Print document and get job number
            m_bJobNr = C56Sdk_DmPrintDocument(hDocument);

            //-- End of document
            C56Sdk_DmEndDocument(hDocument);

            //-- Increase ticket number if auto 
            if (fAuto) 
               this->m_dwTicketNr++;

            //-- Decrease nr of tickes if auto
            if (fAuto) 
               this->m_dwNrOfTickets--;
         }
         else {
 
            //-- Update printing status 
            this->m_strError.Format("STARTING DOCUMENT FAILED");   
         }
      }
      else {

         //-- Update printing error 
         this->m_strError.Format("PRINTER SPOOLER NON EMPTY");  
      }
   }
   else {

      //-- Update printing status 
      this->m_strStatus.Format("\nTICKET PRINTOUT COMPLETED");  
   }

   //-- Update dialog items
   this->SetDlgItems();
}
