/**********************************************************************\
**
**                         Copyright (c) 2004-2017 by
**                          HENGSTLER GmbH, Aldingen
**
**      This  software  is furnished under a licence and may be used and
**      copied only in accordance with the terms  of  such  licence  and
**      with the inclusion of the above copyright notice.  This software
**      or any other copies thereof may not  be  provided  or  otherwise
**      made  available  to any other person.  No title to and ownership
**      of the software is hereby transferred.
**
**      The  information  in  this software is subject to change without
**      notice and should not be construed as a commitment by Hengstler.
**
**  Module:      C56SdkDemo08DlgMain.cpp
**  Facility:    S684002 - Hengstler C56 Printer Driver for Windows XP/7/8/10
**  Component:   C56 SDK Demo Application 08 (C56SdkDemo08)
**  Date         2017APR27
**  Environment: Windows XP/7/8/10
**  Compiler:    Microsoft Visual Studio 2015
**  Abstract:    Implementation of main-dialog of SDK C-56 Demo 08
**  Notes:       -
**
\**********************************************************************/

/*
** INCLUDES
*/

#include "stdafx.h"

#include "..\..\Libraries\C56SDK.h"  /* SDK */

#include "C56SdkDemo08.h"
#include "C56SdkDemo08DlgMain.h"

/*
** COMPILER
*/

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

/*
** MESSAGE MAP
*/

BEGIN_MESSAGE_MAP(CC56SdkDemo08DlgMain, CDialog)
   ON_WM_SYSCOMMAND()
   ON_WM_PAINT()
   ON_WM_QUERYDRAGICON()
	ON_WM_TIMER()
   //}}AFX_MSG_MAP
   ON_BN_CLICKED(IDC_BUTTON_PRINTOUT, OnBnPrintout)
   ON_BN_CLICKED(IDC_BUTTON_SAVE, OnBnSave)
   ON_BN_CLICKED(IDC_BUTTON_LOAD, OnBnLoad)
END_MESSAGE_MAP()

//******************************************************************************************
// METHOD:   CC56SdkDemo08DlgMain::CC56SdkDemo08DlgMain
// ABSTRACT: Object constructor
//******************************************************************************************

CC56SdkDemo08DlgMain::CC56SdkDemo08DlgMain
   (CWnd* pParent /*=NULL*/)
:  CDialog(CC56SdkDemo08DlgMain::IDD, pParent)
{
   //-- Set member icon
   m_hIcon = AfxGetApp()->LoadIcon(IDR_C56);
}

//******************************************************************************************
// METHOD:   CC56SdkDemo08DlgMain::DoDataExchange
// ABSTRACT: Called by the framework to exchange and validate dialog data
//******************************************************************************************

void CC56SdkDemo08DlgMain::DoDataExchange(CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
}

//******************************************************************************************
// METHOD:   CC56SdkDemo08DlgMain::OnInitDialog
// ABSTRACT: This member function is called in response to the WM_INITDIALOG message.
//******************************************************************************************

BOOL CC56SdkDemo08DlgMain::OnInitDialog
   ()
{
   //-- Initialize dialog
   CDialog::OnInitDialog();

   //-- Check IDM_ABOUTBOX for being in system menu
   ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
   ASSERT(IDM_ABOUTBOX < 0xF000);

   //-- Create a timer for update of status
   m_puiTimer = this->SetTimer(1, 100, 0);

   //-- Set C-56 icon for dialog 
   SetIcon(m_hIcon, TRUE);   // Use large symbol verwenden
   SetIcon(m_hIcon, FALSE);  // Use small symbol 

   //-- Invalidate file name
   strcpy_s(m_szFile, sizeof(m_szFile), ""); 

   //-- Setup printer selection combo box with available C-56 printers
   { 
      HANDLE hEnumPrinter; // Handle for enumeration 

      //-- Start access SDK to go through all C-56 printers 
      if ((hEnumPrinter = C56Sdk_PmStartEnumPrinter()) != NULL) {

         LPSTR       lpszPrinter; // Name of printer
         CComboBox * pPrinterList;  // Pointer to combo box item

         //-- Get printer selection combo box item
         if (pPrinterList = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_LIST))) {

            //-- Clear printer selection combo box
            pPrinterList->ResetContent();

            //-- Go through all C-56 printers
            for ( lpszPrinter = C56Sdk_PmEnumPrinter(hEnumPrinter, TRUE)
                ; lpszPrinter != NULL
                ; lpszPrinter = C56Sdk_PmEnumPrinter(hEnumPrinter, FALSE)
                ) {

               //-- Add printer to printer selection combo box 
               pPrinterList->AddString(lpszPrinter);
            }

            //-- Select 1st printer in printer selection combo box
            pPrinterList->SetCurSel(0);
         }
         
         //-- End of enumeration, free handle
         C56Sdk_PmEndEnumPrinter(hEnumPrinter);
      } 
   }
      
   //-- Setup sample text in "tex to print" edit box
   { 
      CEdit * pEscapeCommands; // Pointer to text to print box

      //-- Get text to print (dialog item edit box)
      if ((pEscapeCommands = ((CEdit *) this->GetDlgItem(IDC_EDIT_TEXTTOPRINT))) != NULL) {

          //-- Set sample text to print
          pEscapeCommands->SetWindowText("Enter sample ASCII and HEX text here!");
      }
   }

   //-- Initialization successfull, set focus to first item
   return TRUE;  
}

//******************************************************************************************
// METHOD:   CC56SdkDemo08DlgMain::OnSysCommand
// ABSTRACT: The framework calls this member function when the user selects a command from 
//           the Control menu, or when the user selects the Maximize or the Minimize button.
//******************************************************************************************

void CC56SdkDemo08DlgMain::OnSysCommand
   (UINT nID, LPARAM lParam)
{
   //-- Call original CDialog::OnSysCommand member function
   CDialog::OnSysCommand(nID, lParam);
}

//******************************************************************************************
// METHOD:   CC56SdkDemo08DlgMain::OnPaint
// ABSTRACT: The framework calls this member function when Windows or an application makes a 
//           request to repaint a portion of an application's window.
//******************************************************************************************

void CC56SdkDemo08DlgMain::OnPaint
   () 
{
   //-- Check if dialog is iconic
   if (IsIconic()) {

      CPaintDC dc(this); // Device context for paint 

      //-- Erase background of icon
      SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

      //-- Center symbol in clientr rectangle
      int cxIcon = GetSystemMetrics(SM_CXICON);
      int cyIcon = GetSystemMetrics(SM_CYICON);
      CRect rect;
      GetClientRect(&rect);
      int x = (rect.Width() - cxIcon + 1) / 2;
      int y = (rect.Height() - cyIcon + 1) / 2;

      //-- Draw symbol 
      dc.DrawIcon(x, y, m_hIcon);
   }
   else {

      //-- Call original CDialog::OnPaint member function
      CDialog::OnPaint();
   }
}

//******************************************************************************************
// METHOD:   CC56SdkDemo08DlgMain::OnQueryDragIcon
// ABSTRACT: The framework calls this member function by a minimized (iconic) window that 
//           does not have an icon defined for its class.
//******************************************************************************************

HCURSOR CC56SdkDemo08DlgMain::OnQueryDragIcon
   ()
{
   return static_cast<HCURSOR>(m_hIcon);
}

//******************************************************************************************
// METHOD:   CC56SdkDemo08DlgMain::OnBnPrintout
// ABSTRACT: Handler for printout button being pressed
//******************************************************************************************

void CC56SdkDemo08DlgMain::OnBnPrintout()
{
   CComboBox * pPrinterList;  // Pointer to combo box item
  
   //-- Get printer list (dialog item combo box)
   if ((pPrinterList = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_LIST))) != NULL) {

      INT iPrinterSel; // Selected printer
 
      //-- Determine selected printer
      if ((iPrinterSel = pPrinterList->GetCurSel()) != CB_ERR) {
 
         LPSTR lpszPrinter = NULL;  // Printer name

         //-- Allocate memory for this printer and get name from combo box
         if (    ((lpszPrinter = (LPSTR) malloc(pPrinterList->GetLBTextLen(iPrinterSel)+1)) != NULL) 
              && (pPrinterList->GetLBText(iPrinterSel, lpszPrinter) != CB_ERR)
            ) {

            HANDLE hDocument; // Handle to document 
 
            //-- Start this document
            if ((hDocument = C56Sdk_DmStartDocument(lpszPrinter, "C56SdkDemo08")) != NULL) {

               //-- Add text to print to document
               { 
                  CEdit  * pEscapeCommands; // Pointer to text to print box

                  //-- Get text to print (dialog item edit box)
                  if ((pEscapeCommands = ((CEdit *) this->GetDlgItem(IDC_EDIT_TEXTTOPRINT))) != NULL) {

                     PSTR pszEscapeCommands; // String with escape commands

                     //-- Allocate memory for escape commands added \0
                     if ((pszEscapeCommands = (PSTR) malloc(pEscapeCommands->GetWindowTextLength()+1)) != NULL) {

                        //-- Get commands from dialog including \0
                        if (pEscapeCommands->GetWindowText(pszEscapeCommands, (INT) _msize(pszEscapeCommands)) > 0) {

                           //-- Add "text to print" to document data
                           C56Sdk_DcAddEscapeCmd(hDocument, pszEscapeCommands);
                        }
                     
                        //-- Free allocated memory
                        free(pszEscapeCommands); pszEscapeCommands = NULL;
                     }
                  }
               }

               //-- Print document (no job management)
               C56Sdk_DmPrintDocument2(hDocument);

               //-- End of document
               C56Sdk_DmEndDocument(hDocument);
            }
         }

         //-- Free memory allocated for printer name
         if (lpszPrinter) free(lpszPrinter); lpszPrinter = NULL;
      }
   }
}

//******************************************************************************************
// METHOD:   CC56SdkDemo08DlgMain::OnTimer
// ABSTRACT: This member function is called in response to the WM_TIMER message.
//******************************************************************************************

VOID CC56SdkDemo08DlgMain::OnTimer
   (UINT_PTR nIdEvent)
{
   CComboBox * pPrinterList;  // Pointer to combo box item

   //-- Check if event is private
   if (nIdEvent != m_puiTimer) {

      //-- Other dialog timer
      CDialog::OnTimer(nIdEvent);

   }
   else {
   
      //-- Get printer list (dialog item combo box)
      if ((pPrinterList = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_LIST))) != NULL) {

         INT iPrinterSel; // Selected printer

         //-- Determine selected printer
         if ((iPrinterSel = pPrinterList->GetCurSel()) != CB_ERR) {
 
            LPSTR lpszPrinter = NULL;  // Printer name

            //-- Allocate memory for this printer and get name from combo box
            if (    ((lpszPrinter = (LPSTR) malloc(pPrinterList->GetLBTextLen(iPrinterSel)+1)) != NULL) 
                 && (pPrinterList->GetLBText(iPrinterSel, lpszPrinter) != CB_ERR)
               ) {

               DWORD               dwStatus;             // Printer status
               DWORD               dwAgeMs;              // Age of status in ms
               C56_STATUS_RESPONSE eResponse;            // Status response
               int                 siPhTempC;            // Temperature of printhead in C
               BYTE                bJobNr;               // Last confirmed job number
               CHAR                szFwVersion[10+1]=""; // Current firmware version
               
               //-- Get printer status 
               dwStatus = C56Sdk_PmGetPrinterStatus(lpszPrinter, &dwAgeMs);

               //-- Check age of status being < xxx ms
               //.. and expand status
               //.. and check power on flag, which must allways be raised
               if (     (dwAgeMs <= 10000)
                     && C56Sdk_GpExpandStatus(dwStatus, &eResponse, &siPhTempC, NULL, NULL)
                     && ((eResponse & C56SR_PRINTER_POWERON) != 0)
                   ) {

                  //-- Update printer status
                  {  
                     //-- Update dialog items
                     this->GetDlgItem(IDC_RADIO_STATUS_TEMP_ALERT)->EnableWindow(TRUE);
                     this->CheckDlgButton(IDC_RADIO_STATUS_TEMP_ALERT, (eResponse & C56SR_PRINTER_TEMPALERT)    != 0);
                     this->GetDlgItem(IDC_RADIO_STATUS_LAST_PRINTOUT_LOST)->EnableWindow(TRUE);
                     this->CheckDlgButton(IDC_RADIO_STATUS_LAST_PRINTOUT_LOST, (eResponse & C56SR_PRINTER_PRINTOUTLOST) != 0);
                     this->GetDlgItem(IDC_RADIO_STATUS_PAPER_JAM)->EnableWindow(TRUE);
                     this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_JAM, (eResponse & C56SR_PRINTER_PAPERJAM)     != 0);
                     this->GetDlgItem(IDC_RADIO_STATUS_PAPER_LOW)->EnableWindow(TRUE);
                     this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_LOW, (eResponse & C56SR_PRINTER_PAPERLOW)     != 0);
                     this->GetDlgItem(IDC_RADIO_STATUS_PAPER_OUT)->EnableWindow(TRUE);
                     this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_OUT, (eResponse & C56SR_PRINTER_OUTOFPAPER)   != 0);
                     this->GetDlgItem(IDC_RADIO_STATUS_POWER_ON)->EnableWindow(TRUE);
                     this->CheckDlgButton(IDC_RADIO_STATUS_POWER_ON, (eResponse & C56SR_PRINTER_POWERON)      != 0);
                  }

                  //-- Update printer temperature
                  {  
                     char szBuff[20]; // Buffer

                     //-- Update dialog items
                     sprintf_s(szBuff, sizeof(szBuff),"%-3.3d", siPhTempC);
                     this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C)->EnableWindow(TRUE);
                     this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C)->SetWindowText(szBuff);
                     this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C_TXT)->EnableWindow(TRUE);
                     sprintf_s(szBuff, sizeof(szBuff),"%-3.3d", (signed int) ((siPhTempC * 1.8) + 32));
                     this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F)->EnableWindow(TRUE);
                     this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F)->SetWindowText(szBuff);
                     this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F_TXT)->EnableWindow(TRUE);
                  }

                  //-- Update printer hex code
                  {  
                     char szBuff[20]; // Buffer
                     
                     //-- Update dialog items
                     sprintf_s(szBuff, sizeof(szBuff),"[%2.2X][%2.2X][%2.2X][%2.2X]"
                             , (WORD) (((BYTE *) &dwStatus)[0])
                             , (WORD) (((BYTE *) &dwStatus)[1])
                             , (WORD) (((BYTE *) &dwStatus)[2])
                             , (WORD) (((BYTE *) &dwStatus)[3])
                             );
                     this->GetDlgItem(IDC_STATIC_STATUS_HEX)->EnableWindow(TRUE);
                     this->GetDlgItem(IDC_STATIC_STATUS_HEX)->SetWindowText(szBuff);
                  }

                  //-- Update firmware version
                  {   
                     //-- Enable dialog items
                     this->GetDlgItem(IDC_STATIC_FW_VERSION_TXT)->EnableWindow(TRUE);
                     this->GetDlgItem(IDC_STATIC_FW_VERSION)->EnableWindow(TRUE);

                     //-- Get firmware version
                     if (C56Sdk_PmGetFwVersion(lpszPrinter, szFwVersion, sizeof(szFwVersion)) && szFwVersion[0]) 
                          this->GetDlgItem(IDC_STATIC_FW_VERSION)->SetWindowText(szFwVersion);
                     else this->GetDlgItem(IDC_STATIC_FW_VERSION)->SetWindowText("V-.--");
                     
                  }
               }
               else {

                  //-- Invalidate Printer status
                  {  
                     //-- Update dialog items
                     this->GetDlgItem(IDC_RADIO_STATUS_TEMP_ALERT)->EnableWindow(FALSE);
                     this->CheckDlgButton(IDC_RADIO_STATUS_TEMP_ALERT, FALSE);
                     this->GetDlgItem(IDC_RADIO_STATUS_LAST_PRINTOUT_LOST)->EnableWindow(FALSE);
                     this->CheckDlgButton(IDC_RADIO_STATUS_LAST_PRINTOUT_LOST, FALSE);
                     this->GetDlgItem(IDC_RADIO_STATUS_PAPER_JAM)->EnableWindow(FALSE);
                     this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_JAM, FALSE);
                     this->GetDlgItem(IDC_RADIO_STATUS_PAPER_LOW)->EnableWindow(FALSE);
                     this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_LOW, FALSE);
                     this->GetDlgItem(IDC_RADIO_STATUS_PAPER_OUT)->EnableWindow(FALSE);
                     this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_OUT, FALSE);
                     this->GetDlgItem(IDC_RADIO_STATUS_POWER_ON)->EnableWindow(FALSE);
                     this->CheckDlgButton(IDC_RADIO_STATUS_POWER_ON, FALSE);
                  }

                  //-- Invalidate printer temperature
                  {  
                     //-- Update dialog items
                     this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C)->EnableWindow(FALSE);
                     this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C)->SetWindowText("-");
                     this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C_TXT)->EnableWindow(FALSE);
                     this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F)->EnableWindow(FALSE);
                     this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F)->SetWindowText("-");
                     this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F_TXT)->EnableWindow(FALSE);
                  }
   
                  //-- Invalidate printer hex code
                  {  
                     //-- Update dialog items
                     this->GetDlgItem(IDC_STATIC_STATUS_HEX)->EnableWindow(FALSE);
                     this->GetDlgItem(IDC_STATIC_STATUS_HEX)->SetWindowText("[??][??][??][??]");
                  }

                  //-- Invalidate firmware version
                  {  
                     //-- Update dialog items
                     this->GetDlgItem(IDC_STATIC_FW_VERSION_TXT)->EnableWindow(FALSE);
                     this->GetDlgItem(IDC_STATIC_FW_VERSION)->EnableWindow(FALSE);
                     this->GetDlgItem(IDC_STATIC_FW_VERSION)->SetWindowText("V-.--");
                  }
               }

               //-- Get last confirmed job number 
               bJobNr = C56Sdk_PmGetJobNrConfirmed(lpszPrinter);

               //-- Update job number
               if (bJobNr <= 30) {

                  CHAR szBuff[20];   // Buffer

                  //-- Update dialog items
                  this->GetDlgItem(IDC_STATIC_JOB_NR_TXT)->EnableWindow(TRUE);
                  this->GetDlgItem(IDC_STATIC_JOB_NR)->EnableWindow(TRUE);
                  sprintf_s(szBuff, sizeof(szBuff),"%2.2d", bJobNr);
                  this->GetDlgItem(IDC_STATIC_JOB_NR)->SetWindowText(szBuff);
               }
               else {

                  //-- Invalidate dialog items
                  this->GetDlgItem(IDC_STATIC_JOB_NR_TXT)->EnableWindow(FALSE);
                  this->GetDlgItem(IDC_STATIC_JOB_NR)->EnableWindow(FALSE);
                  this->GetDlgItem(IDC_STATIC_JOB_NR)->SetWindowText("--");
               }
            }
 
            //-- Free allocated memory
            if (lpszPrinter) free(lpszPrinter); lpszPrinter = NULL; 
            
         }
      }    
   }
}

//******************************************************************************************
// METHOD:   CC56SdkDemo08DlgMain::OpenFileName
// ABSTRACT: Handles file selection dialog for save/open file
//******************************************************************************************

BOOL CC56SdkDemo08DlgMain::OpenFileName
   (HWND hWnd, PSTR pszFile, INT nFile, PSTR pszTile, PSTR pszFilter, BOOL fSave)
{
   BOOL fResult = FALSE;                  // Result of this function
   OPENFILENAME  sOpenFileName;           // OpenFileName structure
   
   //-- Initialize OPENFILENAME structure
   ZeroMemory(&sOpenFileName, sizeof(sOpenFileName));
   sOpenFileName.lStructSize     = sizeof(sOpenFileName);
   sOpenFileName.hwndOwner       = hWnd;
   sOpenFileName.lpstrTitle      = pszTile;
   sOpenFileName.lpstrFile       = pszFile;
   sOpenFileName.nMaxFile        = nFile;
   sOpenFileName.lpstrFilter     = pszFilter;
   sOpenFileName.nFilterIndex    = 1;
   sOpenFileName.lpstrFileTitle  = NULL;
   sOpenFileName.nMaxFileTitle   = 0;
   sOpenFileName.lpstrInitialDir = NULL;
   sOpenFileName.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

   //-- Display the open/saev file name dialog box. 
   fResult = (fSave ? GetSaveFileNameA(&sOpenFileName) : GetOpenFileNameA(&sOpenFileName));

   //-- Return result 
   return(fResult);
}

//******************************************************************************************
// METHOD:   CC56SdkDemo08DlgMain::OnBnSave
// ABSTRACT: Saves escape commands to a file
//******************************************************************************************

void CC56SdkDemo08DlgMain::OnBnSave
   ()
{
   CComboBox  * pPrinterList;               // Pointer to combo box item

   //-- Get name of save file 
   if (this->OpenFileName(this->GetSafeHwnd(), m_szFile, sizeof(m_szFile), "SELECT C-56 ESCAPE COMMANDS FILE TO SAVE INTO ...", "C-56 ESCAPE COMMANDS (*.C56EC)\0*.C56EC\0All(*.*)\0*.*\0", TRUE)) {

      //-- Split this path and add extension if not already done
      {
         char szDrive[_MAX_DRIVE];
         char szDir[_MAX_DIR];
         char szFname[_MAX_FNAME];
         char szExt[_MAX_EXT];

         //-- Split path and set extension to C56EC if no extension set
         _splitpath_s(m_szFile, szDrive, sizeof(szDrive), szDir, sizeof(szDir), szFname, sizeof(szFname), szExt, sizeof(szExt));
         if (!strlen(szExt)) strcpy_s(szExt, sizeof(szExt), "C56EC");
         _makepath_s(m_szFile, sizeof(m_szFile), szDrive, szDir, szFname, szExt);
      }
      
      //-- Get printer list (dialog item combo box)
      if ((pPrinterList = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_LIST))) != NULL) {

         INT iPrinterSel; // Selected printer
 
         //-- Determine selected printer
         if ((iPrinterSel = pPrinterList->GetCurSel()) != CB_ERR) {
 
            LPSTR lpszPrinter = NULL;  // Printer name

            //-- Allocate memory for this printer and get name from combo box
            if (    ((lpszPrinter = (LPSTR) malloc(pPrinterList->GetLBTextLen(iPrinterSel)+1)) != NULL) 
                 && (pPrinterList->GetLBText(iPrinterSel, lpszPrinter) != CB_ERR)
               ) {

               CEdit  * pEscapeCommands; // Pointer to escape commands box

               //-- Get escape commands  (dialog item edit box)
               if ((pEscapeCommands = ((CEdit *) this->GetDlgItem(IDC_EDIT_TEXTTOPRINT))) != NULL) {

                  PSTR pszEscapeCommands;  // String with escape commands

                  //-- Allocate memory for escape commands added \0
                  if ((pszEscapeCommands = (PSTR) malloc(pEscapeCommands->GetWindowTextLength()+1)) != NULL) {

                    //-- Get commands from dialog including \0
                     if (pEscapeCommands->GetWindowText(pszEscapeCommands, (INT) _msize(pszEscapeCommands)) > 0) {

                        CFile cFile; // File with escape commands

                        //-- Open file, create if required and truncate if available
                        if (cFile.Open( m_szFile, CFile::modeCreate | CFile::modeWrite, NULL)) {

                           //-- Write to the file
                           cFile.Write(pszEscapeCommands, (INT) _msize(pszEscapeCommands)-1);

                           //-- Close the file
                           cFile.Close();
                        }
                     }
    
                     //-- Free alloced memory
                     free(pszEscapeCommands); pszEscapeCommands = NULL;
                  }
               }

               //-- Free allocated memory
               if (lpszPrinter) free(lpszPrinter); lpszPrinter = NULL; 
            }
         }
      }
   }
}

//******************************************************************************************
// METHOD:   CC56SdkDemo08DlgMain::OnBnLoad
// ABSTRACT: Loads escape commands from a file
//******************************************************************************************

void CC56SdkDemo08DlgMain::OnBnLoad
   ()
{
   CComboBox  * pPrinterList;               // Pointer to combo box item

   //-- Get name of save file 
   if (this->OpenFileName(this->GetSafeHwnd(), m_szFile, sizeof(m_szFile), "SELECT C-56 ESCAPE COMMANDS FILE TO LOAD FROM ...", "C-56 ESCAPE COMMANDS (*.C56EC)\0*.C56EC\0All(*.*)\0*.*\0", FALSE)) {

      //-- Get printer list (dialog item combo box)
      if ((pPrinterList = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_LIST))) != NULL) {

         INT iPrinterSel; // Selected printer
 
         //-- Determine selected printer
         if ((iPrinterSel = pPrinterList->GetCurSel()) != CB_ERR) {
 
            LPSTR lpszPrinter = NULL;  // Printer name

            //-- Allocate memory for this printer and get name from combo box
            if (    ((lpszPrinter = (LPSTR) malloc(pPrinterList->GetLBTextLen(iPrinterSel)+1)) != NULL) 
                 && (pPrinterList->GetLBText(iPrinterSel, lpszPrinter) != CB_ERR)
               ) {

               CFile cFile;  // File with escape commands

               //-- Open file, create if required and truncate if available
               if (cFile.Open( m_szFile, CFile::modeRead | CFile::shareDenyWrite, NULL)) {

                  PSTR pszEscapeCommands = NULL;

                  //-- Allocate memory for escape commands and get commands added \0
                  if ((pszEscapeCommands = (PSTR) malloc((size_t) (cFile.GetLength()+1))) != NULL) {

                     //-- Read escape commands from the file without \0;
                     if (cFile.Read(pszEscapeCommands, (INT) _msize(pszEscapeCommands) - 1) == _msize(pszEscapeCommands) - 1) {

                        CEdit  * pEscapeCommands; // Pointer to escape commands box

                        //-- Terminate this commands 
                        pszEscapeCommands[_msize(pszEscapeCommands)-1] = '\0';

                        //-- Get escape commands  (dialog item edit box)
                        if ((pEscapeCommands = ((CEdit *) this->GetDlgItem(IDC_EDIT_TEXTTOPRINT))) != NULL) {

                            //-- Set escape commands
                            pEscapeCommands->SetWindowText(pszEscapeCommands);
                        }
                     }

                     //-- Free allocated memory
                     free(pszEscapeCommands); pszEscapeCommands = NULL;
                  }    

                  //-- Close the file
                  cFile.Close();
               }

               //-- Free allocated memory
               if (lpszPrinter) free(lpszPrinter); lpszPrinter = NULL; 
            }
         }
      }
   }
}

